;Tilemap routine for FF1. Alpha version, 8 revisions.
;Left and right supported along 256 tiles <-->.
;Handlename: Iambian Zenith
;
;Credits:
;
;Sean McLaughlin for his help with doing bcall(_cpHLDE) without the bcall.
;Patai Gergely and Joe Pemberton for some programming support. (saving registers)
;Patai Gergely, again, for his great ASM tutorial. I would have given up ASM long ago w/o it.
;Shane (on the message boards, he's know as Nindoja) for his support with loading 2 bytes to register pairs.
;
;
;
;
;I'm being extravagent with the credits, but if you find that your name should be here, then feel free to
;complain. I don't mind resubmitting the same program just becuase it needs a new name...
;(this will give me time to improve upon the current routines :-)
;
;
;
;Testing phases:
;(X) Tilemap displaying
;(X) Movement L/R testing
;(X) Movement (no adding) U/D
;(X) Forward decompression
;(X) Backward decompression
;(X) Real-time decompression
;(X) Movement (with adding) U/D
;(X) Support for external data
;(-) Support for external data in FlashROM (won't happen. Too slow. 'bout 100 extra CCs per byte...)
;(-) Greyscale addon
;(1) 14% done: Completion of the world map: data area
;(-) Sprite addon
;(-) Detailing routine (smooth the rough edges)
;(-) Detailing routine (adds the castles and hotspots...)
;(-) smooth scrolling
;(-) variable speed
;(2) further optimiztions and addons
;
;
#include "ti83plus.inc"
;
datawrite	equ	appbackupscreen        ;this is where the tilemap routine will write all of its data in
dataget		equ	appbackupscreen+2      ;\ This is the area where the tilemap routine will read its data from.
                                               ;|Replace all references to "ld de,(sec0)" with "ld de,(dataget)" if and
                                               ;|only if it required to make the buffer "externally" supported. (i.e. code
                                               ;/in FlashAPP, data needs to be in RAM...)
datastart	equ	appbackupscreen+4      ;External data support. This is where the data area of the tiles start
dataend		equ	appbackupscreen+6      ;This is where the data area ends.
Buffer		equ	appbackupscreen+8      ;Buf info here. Adr here points to table of addresses. Mathiness done already.
xpos		equ	appbackupscreen+12     ;x position of character on the map.
ypos		equ	appbackupscreen+13     ;y position of character on the map. Associates with Tbuf and Bbuf.
flashrompage	equ	appbackupscreen+14     ;reading from FlashROM. This number will be set to 0 in demo version.
stackpointer	equ	appbackupscreen+16     ;saves the stack information. This is used until we know it's not the stack.
var1		equ	appbackupscreen+18     ;variable for temporary storage.
var2		equ	appbackupscreen+20     ;variable for temporary storage.
var3		equ	appbackupscreen+22     ;variable for temporary storage.
;
;
;
;
;
.org userMem-2                                 ;RAM position to where the program should start. -2, to accomodate the next statement.
.db $bb,$6d                                    ;AsmPrgm token. Tells the OS that THIS IS AN ASM PROGRAM, NOT BASIC!

 bcall(_clrLCDfull)
 ld hl,$0000
 ld (currow),hl
 ld hl,str0
 bcall(_PutS)



 ld hl,$0000
 ld (Buffer),hl        ;to clean this area before writing. Edit this after implementing so that it reflects "where last was..."

;----------------------------
;Load correct buffer before calling this
;program. One will be provided for testing
;here, but needs to be deleted before 
;application phase is begin.
;----------------------------\
 ld hl,plotsscreen        ; ||
 ld (datawrite),hl        ; ||
 ld hl,sec0               ; ||
 ld (dataget),hl          ; ||
;----------------------------|
;----------------------------/
 
 ld hl,$0000
 ld (currow),hl
 ld hl,str1
 bcall(_PutS)

 ld hl,program
 rst 20h                          ;equivalent to "bcall(_Mov9toOp1)"
 bcall(_ChkFindSym)
 jp nc,noerror

error:

 ld hl,$0000
 ld (currow),hl
 ld hl,fail
 bcall(_PutS)
 ret
 
noerror:
 ld a,b
 or a
 jp nz,error  ;the data should NOT be in FlashROM.

 inc de     ;got offset of the data and skipping for...
 inc de     ;increase by two for programs...
 inc de     ;AsmPrgm instruction
 inc de     ;AsmPrgm instruction
 inc de     ;ret instruction, prevents execution from homescreen.
 inc de     ;offset to use program with... just there, but needs to be skipped over...
 
 ld (var1),de
  
 call initializelongbuffer       ;fills in the data table. Will use the tilemap buffer for its dummywritebuffer.

 call staticdecompression        ;Writes in data for the tilemap. Later on, may use data collected from exit town and call above -/^


 jp tilemove

programstart:
 call fastCopy
 ld a,$ff
 out (1),a
 ld a,$fe
 out (1),a
 in a,(1)
 cp $fd
 jp z,moveleft
 cp $fb
 jp z,moveright
 cp $f7
 jp z,moveup
 cp $fe
 jp z,movedown
 ld a,$ff
 out (1),a
 ld a,$fd
 out (1),a
 in a,(1)
 cp $bf
 ret z         ;quit if clear is pressed
 jr programstart

movedown:

 ld hl,sect
 ld de,secovrt
 ld bc,endsec-sect
 ldir

 ld hl,Buffer
 inc (hl)
 
 jr activedecompression

moveup:

 ld hl,secsubb     ;starting postion of begin loading data
 ld de,endsec      ;writing backwards, so that we may move up
 ld bc,endsec-sect ;distance to set. All-2
 lddr
 
 ld hl,Buffer
 dec (hl)
 
activedecompression:

 ld de,addressdata ;starting point on the table.
 ld a,(Buffer)     ;saved number is recalled (y position)
 ld h,$0           ;\| LD HL,A and resets H
 ld l,a            ;/
 sla l             ;| hl:=2*hl
 rlc h             ;/
 add hl,de         ;OFFSET+BUFFERADDRESS= the number that is needed.
 ld e,(hl)         ;\
 inc hl            ; \|getting number from table. storing it to register pair DE.
 ld d,(hl)         ;_/|
 ex de,hl          ;DE holds the address of the source. Decompression needs this in HL.
 ld de,secovrt     ;Address to start loading the data in.
 call fdecompress  ;decompression starts.
 
 ld de,addressdata ;starting point on the table.
 ld a,(Buffer)     ;saved number is recalled (y position)
 add a,11          ;to shunt the number down to the bottom buffer. Number resets if offset is too much (255+2=1). Carry is not recorded.
 ld h,$0           ;\| LD HL,A and resets H
 ld l,a            ;/
 sla l             ;| hl:=2*hl
 rlc h             ;/
 add hl,de         ;OFFSET+BUFFERADDRESS= the number that is needed.
 ld e,(hl)         ;\
 inc hl            ; \|getting number from table. storing it to register pair DE.
 ld d,(hl)         ;_/|
 ex de,hl          ;DE holds the address of the source. Decompression needs this in HL.
 ld de,secsubb     ;Address to start loading the data. Forwards.
 call fdecompress
 
 jp tilemove

moveleft:

 ld hl,(dataget)
 ld de,sec0
 ld a,l
 cp e      ;seeing if L and E are the same. If Z set, we reached end of world.
 dec hl    ;altering register pairs does not alter the flags.
 jr nz,skipnum
 inc h
skipnum:           ;part of routine shared by both right and left-going sides. Size optimization.
 ld (dataget),hl
 jr tilemove

moveright:
 ld hl,(dataget)
 inc hl
 ld de,sec0
 ld a,l
 cp e
 jr nz,skipnum
 dec h
 jr skipnum

;====================================================================================================
; This is the current tilemap routine that I am using. This tilemap requires that the map is exactly
;256 units wide, because the row-down testing tests for a change in the MSB of a register pair.
;Shunting the map downwards also increases or decreases on the MSB of the register pair instead of
;adding or subtracting to the LSB. I see this approach as faster, but if anyone can optimize this
;current routine (which I know can be done... look how ineffective the code here is!), I'd appreciate
;the efforts.
;====================================================================================================
tilemove:

 di                  ;-|prevent greyscale interrupt (thus far) from reading from the graph buffers while this is writing.
                     ;/|Not sure whether this is good or not... But this is going to be messed around with.

 ld de,(datawrite)   ;destination
 ld hl,(dataget)     ;tilemap data source.

 ld b,8              ;should be 8 rows...

columnloop:

 push bc
 push hl
 push de

 ld b,12             ;12 columns.
rowloop:

 push de
 push hl
 push bc

 ld c,(hl)     ;loading offset of the tilemap
 ld b,$0       ;resetting b and having c hold it
 ld hl,bitmap  ;first part
 sla c         ;performing bc*8.
 rl b
 sla c
 rl b
 sla c
 rl b
 add hl,bc     ;loading the offset.

 ld b,$8      ;there should be 8 loops, one loop for each byte in a tile.

 ex de,hl

tileloop:
 ld a,(de)  ; hl is the postion in the bitmap
 ld (hl),a
 inc de
 push bc
 ld bc,12
 add hl,bc
 pop bc
 djnz tileloop

 pop bc
 pop hl

 inc hl
 ld de,sec0
 ld a,l
 cp e              ;checking if L and E are the same. If so, we have something on the edge of world
 jr nz,noright     ;quit now if we are not at the edge of the world.
 dec h             ;returns back to starting point. Remember, world is 256 tiles wide!
noright:

 pop de

 inc de       ;increase to next position on screen map <-->

 djnz rowloop

 pop de
 pop hl

 ld bc,96
 ex de,hl            ;address is being kept by DE, but processor can only add to HL
 add hl,bc           ;
 ex de,hl            ;exchange back
 inc h               ;increase MSB of hl, for total row movement downward

 pop bc

 djnz columnloop

 ei
 jp programstart    ;The fastdraw routine is in the keyboard reading loop. There is a reason for this...

;====================================================================================================
;
;
;
;
;====================================================================================================
 


fastCopy:
	di
	exx
	ld	a,$80				; 7
	out	($10),a				; 11
	ld	hl,plotsscreen-12-(-(12*64)+1)	; 10
	ld	a,$20				; 7
	ld	c,a				; 4
	inc	hl				; 6 waste
	dec	hl				; 6 waste
fastCopyAgain:
	ld	b,64				; 7
	inc	c				; 4
	ld	de,-(12*64)+1			; 10
	out	($10),a				; 11
	add	hl,de				; 11
	ld	de,10				; 10
fastCopyLoop:
	add	hl,de				; 11
	inc	hl				; 6 waste
	inc	hl				; 6 waste
	inc	de				; 6
	ld	a,(hl)				; 7
	out	($11),a				; 11
	dec	de				; 6
	djnz	fastCopyLoop			; 13/8
	ld	a,c				; 4
	cp	$2B+1				; 7
	jr	nz,fastCopyAgain		; 10/1
	exx
	ei
	ret

initializelongbuffer:   ;DE is already set from the bcall(_ChkFindSym).

 ld hl,$0000
 ld (currow),hl
 ld hl,str2
 bcall(_PutS)
 ld de,(var1)
 ld b,$0
 ld hl,addressdata
 
initializelongbufferloop:

 push bc              ;to save the loop information. fdecompress destroys BC.
 ld (hl),e            ;\
 inc hl               ; \| Saving current address position on compressed map to the table
 ld (hl),d            ; /|
 inc hl               ;/
 push hl              ;saving table position information.
 ex de,hl             ;DE holds the the address. Needed to continue on, and is placed in reg. hl. New DE is trashed.
 ld de,sec0           ;dummy write
 call fdecompress
 ex de,hl             ;restores position. DE is now the address again.
 pop hl	              ;HL is restored, and the loop can continue on.
 pop bc               ;restores the loop information long enough to continue looping.
 djnz initializelongbufferloop
 ret

staticdecompression:   ;this is for test purposes only. This is to initialize the tilemap for the top beginnings.

 ld hl,(var1)
 ld de,secovrt
 ld b,12
 
staticdecompressionloop:
 push bc
 call fdecompress        ;x12
 pop bc
 djnz staticdecompressionloop
 ret


;======================================================================================================
;******************************************************************************************************
;-------------------- Forward moving decompression routine (tested. Works okay.) ----------------------
;******************************************************************************************************
;======================================================================================================
;ARG: HL=source
;     DE=destination
;Notes: B is the subloop counter. C is the total loop counter.
;       Carry is set when addition sends result over edge. See below.
;
fdecompress:

 ld c,0       ;For the counter, register C needs to be reset.
 ld a,(hl)
 or a
 jr z,fspecialtile  ;identifies as all water. No further looping is necessary
 
 
fdecompressloop:

 ld a,(hl)    ;above was taken out of the loop to make it faster. This is required to run the loop.

 push bc      ;since register C is being destroyed in this subloop...

 ld b,a       ;the number just viewed was the loop counter.
 ld c,b       ;hold the value of register B for a while (to be added to total count)
 inc hl       ;shunt register HL upward to view the number being copied.
 ld a,(hl)    ;gotta know what that number is...
 sub $ef      ;to get the offset correct (from high number lowest starting at 0)
 inc hl       ;get to the next number for next iteration.

flooparound:

 ld (de),a
 inc de
 djnz flooparound
 ld a,c       ;restore value back to register B, and in turn, back to A

 pop bc       ;restore register C.

 add a,c      ;adds current counter by current result. Carry set if too much (meaning, stop loop!!!)
 ld c,a       ;send back result to register C. Does not alter the flags (meaning carry is not affected)

 jr nc,fdecompressloop
 ret          ;to end the call to decompressloop.

fspecialtile:
 inc hl       ;|_To set next number second time this loop is run (so no external data is needed!)
 inc hl       ;|/
 ld a,3       ;number that corresponds to seawater.
 ld b,0       ;to make djnz loop run 256 times.

flooparound2:

 ld (de),a
 inc de
 djnz flooparound2
 ret          ;to end the call to decompressloop.

;----------------------------------------------------------------------------------------
; 
; There was a backward-moving decompression subroutine, but it was no longer needed. 
;And, thus, the subroutine was taken out. It was taking too much memory (50 bytes or so)
; 
;----------------------------------------------------------------------------------------
 
 

;======================================================================================================
;******************************************************************************************************
;-------------------- Uncompressed raw data. Used when decompression routine is NOT working -----------
;******************************************************************************************************
;======================================================================================================


secovrt:
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
sect:
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
sec0:
.db $02,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$02
sec1:
.db $02,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $01,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$02
sec2:
.db $02,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00
.db $00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$02
sec3:
.db $02,$01,$01,$00,$00,$00,$00,$00,$01,$01,$00,$00,$00,$00,$01,$01,$01,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$02
sec4:
.db $02,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$02
sec5:
.db $02,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$02
sec6:
.db $02,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$02
sec7:
.db $02,$00,$00,$00,$00,$01,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$01,$00,$00,$00,$00,$01,$00,$00,$00,$00,$02
secb:
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
secsubb:
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
endsec:
.ds 2

addressdata:
.ds 256*2+2


;======================================================================================================
;******************************************************************************************************
;-------------------- Data area for all tile information. bitmap=mono gbitmap=greyscale ---------------
;******************************************************************************************************
;======================================================================================================

bitmap:
map00:          ;the "green". Better than nothing.
.db %10100000
.db %00010000
.db %01010010
.db %00000000
.db %01000000
.db %00000000
.db %00000010
.db %01000100
map01:          ;the "grass". Better than nothing.
.db %01000000
.db %01001010
.db %00100000
.db %00100100
.db %00000010
.db %01001001
.db %00100000
.db %00100101
map02:         ;God only knows what this is supposed to represent. Seriously. (desert)
.db %01000000
.db %00001000
.db %00000000
.db %00000010
.db %00100000
.db %00000000
.db %00000100
.db %01000000
map03:            ;(sea) water. Homecooked with fair suggestions. Better when in greyscale (not designed yet)
.db %11010011
.db %01101111
.db %11111111
.db %10110001
.db %01010110
.db %11010111
.db %11101100
.db %11111011
map04:         ;"river water", tile ripped from Joltima
.db %10111111
.db %01011011
.db %11110101
.db %11111111
.db %11011111
.db %10101101
.db %11111010
.db %11111111
map05:         ;"mountains", design based mostly from Joltima's tile.
.db %00001000
.db %00011000
.db %00110110
.db %01100100
.db %11001001
.db %10011101
.db %00110010
.db %00100011
map06:        ;Pointer to castle Corneria (don't draw yet)
.db %
.db %
.db %
.db %
.db %
.db %
.db %
.db %
map07:        ;solid concrete. Just a white path, so don't even bother trying to draw on it.
.db %00000000
.db %00000000
.db %00000000
.db %00000000
.db %00000000
.db %00000000
.db %00000000
.db %00000000
map08:        ;"town". Better in greyscale (neither designed yet).
.db %
.db %
.db %
.db %
.db %
.db %
.db %
.db %
map09:        ;"forest". Was homecooked, though I thought about ripping this tile from Joltima, too.
.db %00000000
.db %00100010
.db %00100010
.db %01010101
.db %01010101
.db %10001000
.db %11111111
.db %00100010
map0a:        ;pointer to castle2. Not drawn. (don't draw)
.db %
.db %
.db %
.db %
.db %
.db %
.db %
.db %
map0b:        ;ditto above. C3 (don't draw)
.db %
.db %
.db %
.db %
.db %
.db %
.db %
.db %
map0c:        ;"waterfall". Had to be homecooked..er.. homeboiled... :) Needs to be drawn.
.db %
.db %
.db %
.db %
.db %
.db %
.db %
.db %
map0d:         ;patch of "thorns". Homecooked.
.db %10010100
.db %10010001
.db %00100100
.db %01001000
.db %01001001
.db %00010001
.db %10011100
.db %10000000
map0e:        ;pointer to the tower. (don't draw)
.db %
.db %
.db %
.db %
.db %
.db %
.db %
.db %
map0f:        ;It's a hole in the ground. It isn't going to get any better in greyscale!
.db %00111100
.db %01111110
.db %11111111
.db %11111111
.db %11111111
.db %11111111
.db %01111110
.db %00111100



;======================================================================================================
;******************************************************************************************************
;--------------------- String data, to give tester the idea of what's going on. -----------------------
;--------------------- But most of the time, it goes too fast for the human eye -----------------------
;******************************************************************************************************
;======================================================================================================

str0:
.db "Begin Program   ",0
str1:
.db "Detecting AppVar",0
str2:
.db "AdrDataBufferR/W",0
str3:
.db "Press enter.    ",0

fail:
.db "Program failure."
.db "Must terminate  "
.db "immediately.    ",0

program:
.db ProtProgObj,"TMAP",0,0,0,0,0

.end

preservedtiles:    ;its here so that it won't affect the code if I compile before readjusting the map.








